# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from .accelerator_type import (
    AcceleratorType,
)
from .annotation import (
    Annotation,
)
from .annotation_spec import (
    AnnotationSpec,
)
from .artifact import (
    Artifact,
)
from .batch_prediction_job import (
    BatchPredictionJob,
)
from .completion_stats import (
    CompletionStats,
)
from .content import (
    Blob,
    Candidate,
    Citation,
    CitationMetadata,
    Content,
    FileData,
    GenerationConfig,
    GroundingChunk,
    GroundingMetadata,
    GroundingSupport,
    LogprobsResult,
    Part,
    SafetyRating,
    SafetySetting,
    SearchEntryPoint,
    Segment,
    VideoMetadata,
    HarmCategory,
)
from .context import (
    Context,
)
from .custom_job import (
    ContainerSpec,
    CustomJob,
    CustomJobSpec,
    PythonPackageSpec,
    Scheduling,
    WorkerPoolSpec,
)
from .data_item import (
    DataItem,
)
from .data_labeling_job import (
    ActiveLearningConfig,
    DataLabelingJob,
    SampleConfig,
    TrainingConfig,
)
from .dataset import (
    Dataset,
    ExportDataConfig,
    ExportFilterSplit,
    ExportFractionSplit,
    ImportDataConfig,
)
from .dataset_service import (
    CreateDatasetOperationMetadata,
    CreateDatasetRequest,
    CreateDatasetVersionOperationMetadata,
    CreateDatasetVersionRequest,
    DataItemView,
    DeleteDatasetRequest,
    DeleteDatasetVersionRequest,
    DeleteSavedQueryRequest,
    ExportDataOperationMetadata,
    ExportDataRequest,
    ExportDataResponse,
    GetAnnotationSpecRequest,
    GetDatasetRequest,
    GetDatasetVersionRequest,
    ImportDataOperationMetadata,
    ImportDataRequest,
    ImportDataResponse,
    ListAnnotationsRequest,
    ListAnnotationsResponse,
    ListDataItemsRequest,
    ListDataItemsResponse,
    ListDatasetsRequest,
    ListDatasetsResponse,
    ListDatasetVersionsRequest,
    ListDatasetVersionsResponse,
    ListSavedQueriesRequest,
    ListSavedQueriesResponse,
    RestoreDatasetVersionOperationMetadata,
    RestoreDatasetVersionRequest,
    SearchDataItemsRequest,
    SearchDataItemsResponse,
    UpdateDatasetRequest,
    UpdateDatasetVersionRequest,
)
from .dataset_version import (
    DatasetVersion,
)
from .deployed_index_ref import (
    DeployedIndexRef,
)
from .deployed_model_ref import (
    DeployedModelRef,
)
from .deployment_resource_pool import (
    DeploymentResourcePool,
)
from .deployment_resource_pool_service import (
    CreateDeploymentResourcePoolOperationMetadata,
    CreateDeploymentResourcePoolRequest,
    DeleteDeploymentResourcePoolRequest,
    GetDeploymentResourcePoolRequest,
    ListDeploymentResourcePoolsRequest,
    ListDeploymentResourcePoolsResponse,
    QueryDeployedModelsRequest,
    QueryDeployedModelsResponse,
    UpdateDeploymentResourcePoolOperationMetadata,
    UpdateDeploymentResourcePoolRequest,
)
from .encryption_spec import (
    EncryptionSpec,
)
from .endpoint import (
    DeployedModel,
    Endpoint,
    PredictRequestResponseLoggingConfig,
    PrivateEndpoints,
)
from .endpoint_service import (
    CreateEndpointOperationMetadata,
    CreateEndpointRequest,
    DeleteEndpointRequest,
    DeployModelOperationMetadata,
    DeployModelRequest,
    DeployModelResponse,
    GetEndpointRequest,
    ListEndpointsRequest,
    ListEndpointsResponse,
    MutateDeployedModelOperationMetadata,
    MutateDeployedModelRequest,
    MutateDeployedModelResponse,
    UndeployModelOperationMetadata,
    UndeployModelRequest,
    UndeployModelResponse,
    UpdateEndpointRequest,
)
from .entity_type import (
    EntityType,
)
from .env_var import (
    EnvVar,
)
from .evaluated_annotation import (
    ErrorAnalysisAnnotation,
    EvaluatedAnnotation,
    EvaluatedAnnotationExplanation,
)
from .evaluation_service import (
    BleuInput,
    BleuInstance,
    BleuMetricValue,
    BleuResults,
    BleuSpec,
    CoherenceInput,
    CoherenceInstance,
    CoherenceResult,
    CoherenceSpec,
    EvaluateInstancesRequest,
    EvaluateInstancesResponse,
    ExactMatchInput,
    ExactMatchInstance,
    ExactMatchMetricValue,
    ExactMatchResults,
    ExactMatchSpec,
    FluencyInput,
    FluencyInstance,
    FluencyResult,
    FluencySpec,
    FulfillmentInput,
    FulfillmentInstance,
    FulfillmentResult,
    FulfillmentSpec,
    GroundednessInput,
    GroundednessInstance,
    GroundednessResult,
    GroundednessSpec,
    PairwiseMetricInput,
    PairwiseMetricInstance,
    PairwiseMetricResult,
    PairwiseMetricSpec,
    PairwiseQuestionAnsweringQualityInput,
    PairwiseQuestionAnsweringQualityInstance,
    PairwiseQuestionAnsweringQualityResult,
    PairwiseQuestionAnsweringQualitySpec,
    PairwiseSummarizationQualityInput,
    PairwiseSummarizationQualityInstance,
    PairwiseSummarizationQualityResult,
    PairwiseSummarizationQualitySpec,
    PointwiseMetricInput,
    PointwiseMetricInstance,
    PointwiseMetricResult,
    PointwiseMetricSpec,
    QuestionAnsweringCorrectnessInput,
    QuestionAnsweringCorrectnessInstance,
    QuestionAnsweringCorrectnessResult,
    QuestionAnsweringCorrectnessSpec,
    QuestionAnsweringHelpfulnessInput,
    QuestionAnsweringHelpfulnessInstance,
    QuestionAnsweringHelpfulnessResult,
    QuestionAnsweringHelpfulnessSpec,
    QuestionAnsweringQualityInput,
    QuestionAnsweringQualityInstance,
    QuestionAnsweringQualityResult,
    QuestionAnsweringQualitySpec,
    QuestionAnsweringRelevanceInput,
    QuestionAnsweringRelevanceInstance,
    QuestionAnsweringRelevanceResult,
    QuestionAnsweringRelevanceSpec,
    RougeInput,
    RougeInstance,
    RougeMetricValue,
    RougeResults,
    RougeSpec,
    SafetyInput,
    SafetyInstance,
    SafetyResult,
    SafetySpec,
    SummarizationHelpfulnessInput,
    SummarizationHelpfulnessInstance,
    SummarizationHelpfulnessResult,
    SummarizationHelpfulnessSpec,
    SummarizationQualityInput,
    SummarizationQualityInstance,
    SummarizationQualityResult,
    SummarizationQualitySpec,
    SummarizationVerbosityInput,
    SummarizationVerbosityInstance,
    SummarizationVerbosityResult,
    SummarizationVerbositySpec,
    ToolCallValidInput,
    ToolCallValidInstance,
    ToolCallValidMetricValue,
    ToolCallValidResults,
    ToolCallValidSpec,
    ToolNameMatchInput,
    ToolNameMatchInstance,
    ToolNameMatchMetricValue,
    ToolNameMatchResults,
    ToolNameMatchSpec,
    ToolParameterKeyMatchInput,
    ToolParameterKeyMatchInstance,
    ToolParameterKeyMatchMetricValue,
    ToolParameterKeyMatchResults,
    ToolParameterKeyMatchSpec,
    ToolParameterKVMatchInput,
    ToolParameterKVMatchInstance,
    ToolParameterKVMatchMetricValue,
    ToolParameterKVMatchResults,
    ToolParameterKVMatchSpec,
    PairwiseChoice,
)
from .event import (
    Event,
)
from .execution import (
    Execution,
)
from .explanation import (
    Attribution,
    BlurBaselineConfig,
    Examples,
    ExamplesOverride,
    ExamplesRestrictionsNamespace,
    Explanation,
    ExplanationMetadataOverride,
    ExplanationParameters,
    ExplanationSpec,
    ExplanationSpecOverride,
    FeatureNoiseSigma,
    IntegratedGradientsAttribution,
    ModelExplanation,
    Neighbor,
    Presets,
    SampledShapleyAttribution,
    SmoothGradConfig,
    XraiAttribution,
)
from .explanation_metadata import (
    ExplanationMetadata,
)
from .feature import (
    Feature,
)
from .feature_group import (
    FeatureGroup,
)
from .feature_monitoring_stats import (
    FeatureStatsAnomaly,
)
from .feature_online_store import (
    FeatureOnlineStore,
)
from .feature_online_store_admin_service import (
    CreateFeatureOnlineStoreOperationMetadata,
    CreateFeatureOnlineStoreRequest,
    CreateFeatureViewOperationMetadata,
    CreateFeatureViewRequest,
    DeleteFeatureOnlineStoreRequest,
    DeleteFeatureViewRequest,
    GetFeatureOnlineStoreRequest,
    GetFeatureViewRequest,
    GetFeatureViewSyncRequest,
    ListFeatureOnlineStoresRequest,
    ListFeatureOnlineStoresResponse,
    ListFeatureViewsRequest,
    ListFeatureViewsResponse,
    ListFeatureViewSyncsRequest,
    ListFeatureViewSyncsResponse,
    SyncFeatureViewRequest,
    SyncFeatureViewResponse,
    UpdateFeatureOnlineStoreOperationMetadata,
    UpdateFeatureOnlineStoreRequest,
    UpdateFeatureViewOperationMetadata,
    UpdateFeatureViewRequest,
)
from .feature_online_store_service import (
    FeatureViewDataKey,
    FetchFeatureValuesRequest,
    FetchFeatureValuesResponse,
    NearestNeighborQuery,
    NearestNeighbors,
    SearchNearestEntitiesRequest,
    SearchNearestEntitiesResponse,
    FeatureViewDataFormat,
)
from .feature_registry_service import (
    CreateFeatureGroupOperationMetadata,
    CreateFeatureGroupRequest,
    CreateRegistryFeatureOperationMetadata,
    DeleteFeatureGroupRequest,
    GetFeatureGroupRequest,
    ListFeatureGroupsRequest,
    ListFeatureGroupsResponse,
    UpdateFeatureGroupOperationMetadata,
    UpdateFeatureGroupRequest,
    UpdateFeatureOperationMetadata,
)
from .feature_selector import (
    FeatureSelector,
    IdMatcher,
)
from .feature_view import (
    FeatureView,
)
from .feature_view_sync import (
    FeatureViewSync,
)
from .featurestore import (
    Featurestore,
)
from .featurestore_monitoring import (
    FeaturestoreMonitoringConfig,
)
from .featurestore_online_service import (
    FeatureValue,
    FeatureValueList,
    ReadFeatureValuesRequest,
    ReadFeatureValuesResponse,
    StreamingReadFeatureValuesRequest,
    StructFieldValue,
    StructValue,
    WriteFeatureValuesPayload,
    WriteFeatureValuesRequest,
    WriteFeatureValuesResponse,
)
from .featurestore_service import (
    BatchCreateFeaturesOperationMetadata,
    BatchCreateFeaturesRequest,
    BatchCreateFeaturesResponse,
    BatchReadFeatureValuesOperationMetadata,
    BatchReadFeatureValuesRequest,
    BatchReadFeatureValuesResponse,
    CreateEntityTypeOperationMetadata,
    CreateEntityTypeRequest,
    CreateFeatureOperationMetadata,
    CreateFeatureRequest,
    CreateFeaturestoreOperationMetadata,
    CreateFeaturestoreRequest,
    DeleteEntityTypeRequest,
    DeleteFeatureRequest,
    DeleteFeaturestoreRequest,
    DeleteFeatureValuesOperationMetadata,
    DeleteFeatureValuesRequest,
    DeleteFeatureValuesResponse,
    DestinationFeatureSetting,
    EntityIdSelector,
    ExportFeatureValuesOperationMetadata,
    ExportFeatureValuesRequest,
    ExportFeatureValuesResponse,
    FeatureValueDestination,
    GetEntityTypeRequest,
    GetFeatureRequest,
    GetFeaturestoreRequest,
    ImportFeatureValuesOperationMetadata,
    ImportFeatureValuesRequest,
    ImportFeatureValuesResponse,
    ListEntityTypesRequest,
    ListEntityTypesResponse,
    ListFeaturesRequest,
    ListFeaturesResponse,
    ListFeaturestoresRequest,
    ListFeaturestoresResponse,
    SearchFeaturesRequest,
    SearchFeaturesResponse,
    UpdateEntityTypeRequest,
    UpdateFeatureRequest,
    UpdateFeaturestoreOperationMetadata,
    UpdateFeaturestoreRequest,
)
from .genai_tuning_service import (
    CancelTuningJobRequest,
    CreateTuningJobRequest,
    GetTuningJobRequest,
    ListTuningJobsRequest,
    ListTuningJobsResponse,
)
from .hyperparameter_tuning_job import (
    HyperparameterTuningJob,
)
from .index import (
    Index,
    IndexDatapoint,
    IndexStats,
)
from .index_endpoint import (
    DeployedIndex,
    DeployedIndexAuthConfig,
    IndexEndpoint,
    IndexPrivateEndpoints,
)
from .index_endpoint_service import (
    CreateIndexEndpointOperationMetadata,
    CreateIndexEndpointRequest,
    DeleteIndexEndpointRequest,
    DeployIndexOperationMetadata,
    DeployIndexRequest,
    DeployIndexResponse,
    GetIndexEndpointRequest,
    ListIndexEndpointsRequest,
    ListIndexEndpointsResponse,
    MutateDeployedIndexOperationMetadata,
    MutateDeployedIndexRequest,
    MutateDeployedIndexResponse,
    UndeployIndexOperationMetadata,
    UndeployIndexRequest,
    UndeployIndexResponse,
    UpdateIndexEndpointRequest,
)
from .index_service import (
    CreateIndexOperationMetadata,
    CreateIndexRequest,
    DeleteIndexRequest,
    GetIndexRequest,
    ListIndexesRequest,
    ListIndexesResponse,
    NearestNeighborSearchOperationMetadata,
    RemoveDatapointsRequest,
    RemoveDatapointsResponse,
    UpdateIndexOperationMetadata,
    UpdateIndexRequest,
    UpsertDatapointsRequest,
    UpsertDatapointsResponse,
)
from .io import (
    AvroSource,
    BigQueryDestination,
    BigQuerySource,
    ContainerRegistryDestination,
    CsvDestination,
    CsvSource,
    GcsDestination,
    GcsSource,
    TFRecordDestination,
)
from .job_service import (
    CancelBatchPredictionJobRequest,
    CancelCustomJobRequest,
    CancelDataLabelingJobRequest,
    CancelHyperparameterTuningJobRequest,
    CancelNasJobRequest,
    CreateBatchPredictionJobRequest,
    CreateCustomJobRequest,
    CreateDataLabelingJobRequest,
    CreateHyperparameterTuningJobRequest,
    CreateModelDeploymentMonitoringJobRequest,
    CreateNasJobRequest,
    DeleteBatchPredictionJobRequest,
    DeleteCustomJobRequest,
    DeleteDataLabelingJobRequest,
    DeleteHyperparameterTuningJobRequest,
    DeleteModelDeploymentMonitoringJobRequest,
    DeleteNasJobRequest,
    GetBatchPredictionJobRequest,
    GetCustomJobRequest,
    GetDataLabelingJobRequest,
    GetHyperparameterTuningJobRequest,
    GetModelDeploymentMonitoringJobRequest,
    GetNasJobRequest,
    GetNasTrialDetailRequest,
    ListBatchPredictionJobsRequest,
    ListBatchPredictionJobsResponse,
    ListCustomJobsRequest,
    ListCustomJobsResponse,
    ListDataLabelingJobsRequest,
    ListDataLabelingJobsResponse,
    ListHyperparameterTuningJobsRequest,
    ListHyperparameterTuningJobsResponse,
    ListModelDeploymentMonitoringJobsRequest,
    ListModelDeploymentMonitoringJobsResponse,
    ListNasJobsRequest,
    ListNasJobsResponse,
    ListNasTrialDetailsRequest,
    ListNasTrialDetailsResponse,
    PauseModelDeploymentMonitoringJobRequest,
    ResumeModelDeploymentMonitoringJobRequest,
    SearchModelDeploymentMonitoringStatsAnomaliesRequest,
    SearchModelDeploymentMonitoringStatsAnomaliesResponse,
    UpdateModelDeploymentMonitoringJobOperationMetadata,
    UpdateModelDeploymentMonitoringJobRequest,
)
from .job_state import (
    JobState,
)
from .lineage_subgraph import (
    LineageSubgraph,
)
from .llm_utility_service import (
    ComputeTokensRequest,
    ComputeTokensResponse,
    TokensInfo,
)
from .machine_resources import (
    AutomaticResources,
    AutoscalingMetricSpec,
    BatchDedicatedResources,
    DedicatedResources,
    DiskSpec,
    MachineSpec,
    NfsMount,
    PersistentDiskSpec,
    ResourcesConsumed,
    ShieldedVmConfig,
)
from .manual_batch_tuning_parameters import (
    ManualBatchTuningParameters,
)
from .match_service import (
    FindNeighborsRequest,
    FindNeighborsResponse,
    ReadIndexDatapointsRequest,
    ReadIndexDatapointsResponse,
)
from .metadata_schema import (
    MetadataSchema,
)
from .metadata_service import (
    AddContextArtifactsAndExecutionsRequest,
    AddContextArtifactsAndExecutionsResponse,
    AddContextChildrenRequest,
    AddContextChildrenResponse,
    AddExecutionEventsRequest,
    AddExecutionEventsResponse,
    CreateArtifactRequest,
    CreateContextRequest,
    CreateExecutionRequest,
    CreateMetadataSchemaRequest,
    CreateMetadataStoreOperationMetadata,
    CreateMetadataStoreRequest,
    DeleteArtifactRequest,
    DeleteContextRequest,
    DeleteExecutionRequest,
    DeleteMetadataStoreOperationMetadata,
    DeleteMetadataStoreRequest,
    GetArtifactRequest,
    GetContextRequest,
    GetExecutionRequest,
    GetMetadataSchemaRequest,
    GetMetadataStoreRequest,
    ListArtifactsRequest,
    ListArtifactsResponse,
    ListContextsRequest,
    ListContextsResponse,
    ListExecutionsRequest,
    ListExecutionsResponse,
    ListMetadataSchemasRequest,
    ListMetadataSchemasResponse,
    ListMetadataStoresRequest,
    ListMetadataStoresResponse,
    PurgeArtifactsMetadata,
    PurgeArtifactsRequest,
    PurgeArtifactsResponse,
    PurgeContextsMetadata,
    PurgeContextsRequest,
    PurgeContextsResponse,
    PurgeExecutionsMetadata,
    PurgeExecutionsRequest,
    PurgeExecutionsResponse,
    QueryArtifactLineageSubgraphRequest,
    QueryContextLineageSubgraphRequest,
    QueryExecutionInputsAndOutputsRequest,
    RemoveContextChildrenRequest,
    RemoveContextChildrenResponse,
    UpdateArtifactRequest,
    UpdateContextRequest,
    UpdateExecutionRequest,
)
from .metadata_store import (
    MetadataStore,
)
from .migratable_resource import (
    MigratableResource,
)
from .migration_service import (
    BatchMigrateResourcesOperationMetadata,
    BatchMigrateResourcesRequest,
    BatchMigrateResourcesResponse,
    MigrateResourceRequest,
    MigrateResourceResponse,
    SearchMigratableResourcesRequest,
    SearchMigratableResourcesResponse,
)
from .model import (
    GenieSource,
    LargeModelReference,
    Model,
    ModelContainerSpec,
    ModelGardenSource,
    ModelSourceInfo,
    Port,
    PredictSchemata,
    Probe,
)
from .model_deployment_monitoring_job import (
    ModelDeploymentMonitoringBigQueryTable,
    ModelDeploymentMonitoringJob,
    ModelDeploymentMonitoringObjectiveConfig,
    ModelDeploymentMonitoringScheduleConfig,
    ModelMonitoringStatsAnomalies,
    ModelDeploymentMonitoringObjectiveType,
)
from .model_evaluation import (
    ModelEvaluation,
)
from .model_evaluation_slice import (
    ModelEvaluationSlice,
)
from .model_garden_service import (
    GetPublisherModelRequest,
    PublisherModelView,
)
from .model_monitoring import (
    ModelMonitoringAlertConfig,
    ModelMonitoringObjectiveConfig,
    SamplingStrategy,
    ThresholdConfig,
)
from .model_service import (
    BatchImportEvaluatedAnnotationsRequest,
    BatchImportEvaluatedAnnotationsResponse,
    BatchImportModelEvaluationSlicesRequest,
    BatchImportModelEvaluationSlicesResponse,
    CopyModelOperationMetadata,
    CopyModelRequest,
    CopyModelResponse,
    DeleteModelRequest,
    DeleteModelVersionRequest,
    ExportModelOperationMetadata,
    ExportModelRequest,
    ExportModelResponse,
    GetModelEvaluationRequest,
    GetModelEvaluationSliceRequest,
    GetModelRequest,
    ImportModelEvaluationRequest,
    ListModelEvaluationSlicesRequest,
    ListModelEvaluationSlicesResponse,
    ListModelEvaluationsRequest,
    ListModelEvaluationsResponse,
    ListModelsRequest,
    ListModelsResponse,
    ListModelVersionsRequest,
    ListModelVersionsResponse,
    MergeVersionAliasesRequest,
    UpdateExplanationDatasetOperationMetadata,
    UpdateExplanationDatasetRequest,
    UpdateExplanationDatasetResponse,
    UpdateModelRequest,
    UploadModelOperationMetadata,
    UploadModelRequest,
    UploadModelResponse,
)
from .nas_job import (
    NasJob,
    NasJobOutput,
    NasJobSpec,
    NasTrial,
    NasTrialDetail,
)
from .network_spec import (
    NetworkSpec,
)
from .notebook_euc_config import (
    NotebookEucConfig,
)
from .notebook_execution_job import (
    NotebookExecutionJob,
)
from .notebook_idle_shutdown_config import (
    NotebookIdleShutdownConfig,
)
from .notebook_runtime import (
    NotebookRuntime,
    NotebookRuntimeTemplate,
    NotebookRuntimeType,
)
from .notebook_runtime_template_ref import (
    NotebookRuntimeTemplateRef,
)
from .notebook_service import (
    AssignNotebookRuntimeOperationMetadata,
    AssignNotebookRuntimeRequest,
    CreateNotebookExecutionJobOperationMetadata,
    CreateNotebookExecutionJobRequest,
    CreateNotebookRuntimeTemplateOperationMetadata,
    CreateNotebookRuntimeTemplateRequest,
    DeleteNotebookExecutionJobRequest,
    DeleteNotebookRuntimeRequest,
    DeleteNotebookRuntimeTemplateRequest,
    GetNotebookExecutionJobRequest,
    GetNotebookRuntimeRequest,
    GetNotebookRuntimeTemplateRequest,
    ListNotebookExecutionJobsRequest,
    ListNotebookExecutionJobsResponse,
    ListNotebookRuntimesRequest,
    ListNotebookRuntimesResponse,
    ListNotebookRuntimeTemplatesRequest,
    ListNotebookRuntimeTemplatesResponse,
    StartNotebookRuntimeOperationMetadata,
    StartNotebookRuntimeRequest,
    StartNotebookRuntimeResponse,
    UpdateNotebookRuntimeTemplateRequest,
    UpgradeNotebookRuntimeOperationMetadata,
    UpgradeNotebookRuntimeRequest,
    UpgradeNotebookRuntimeResponse,
    NotebookExecutionJobView,
)
from .openapi import (
    Schema,
    Type,
)
from .operation import (
    DeleteOperationMetadata,
    GenericOperationMetadata,
)
from .persistent_resource import (
    PersistentResource,
    RayLogsSpec,
    RayMetricSpec,
    RaySpec,
    ResourcePool,
    ResourceRuntime,
    ResourceRuntimeSpec,
    ServiceAccountSpec,
)
from .persistent_resource_service import (
    CreatePersistentResourceOperationMetadata,
    CreatePersistentResourceRequest,
    DeletePersistentResourceRequest,
    GetPersistentResourceRequest,
    ListPersistentResourcesRequest,
    ListPersistentResourcesResponse,
    RebootPersistentResourceOperationMetadata,
    RebootPersistentResourceRequest,
    UpdatePersistentResourceOperationMetadata,
    UpdatePersistentResourceRequest,
)
from .pipeline_failure_policy import (
    PipelineFailurePolicy,
)
from .pipeline_job import (
    PipelineJob,
    PipelineJobDetail,
    PipelineTaskDetail,
    PipelineTaskExecutorDetail,
    PipelineTemplateMetadata,
)
from .pipeline_service import (
    BatchCancelPipelineJobsOperationMetadata,
    BatchCancelPipelineJobsRequest,
    BatchCancelPipelineJobsResponse,
    BatchDeletePipelineJobsRequest,
    BatchDeletePipelineJobsResponse,
    CancelPipelineJobRequest,
    CancelTrainingPipelineRequest,
    CreatePipelineJobRequest,
    CreateTrainingPipelineRequest,
    DeletePipelineJobRequest,
    DeleteTrainingPipelineRequest,
    GetPipelineJobRequest,
    GetTrainingPipelineRequest,
    ListPipelineJobsRequest,
    ListPipelineJobsResponse,
    ListTrainingPipelinesRequest,
    ListTrainingPipelinesResponse,
)
from .pipeline_state import (
    PipelineState,
)
from .prediction_service import (
    CountTokensRequest,
    CountTokensResponse,
    DirectPredictRequest,
    DirectPredictResponse,
    DirectRawPredictRequest,
    DirectRawPredictResponse,
    ExplainRequest,
    ExplainResponse,
    GenerateContentRequest,
    GenerateContentResponse,
    PredictRequest,
    PredictResponse,
    RawPredictRequest,
    StreamDirectPredictRequest,
    StreamDirectPredictResponse,
    StreamDirectRawPredictRequest,
    StreamDirectRawPredictResponse,
    StreamingPredictRequest,
    StreamingPredictResponse,
    StreamingRawPredictRequest,
    StreamingRawPredictResponse,
    StreamRawPredictRequest,
)
from .publisher_model import (
    PublisherModel,
)
from .reservation_affinity import (
    ReservationAffinity,
)
from .saved_query import (
    SavedQuery,
)
from .schedule import (
    Schedule,
)
from .schedule_service import (
    CreateScheduleRequest,
    DeleteScheduleRequest,
    GetScheduleRequest,
    ListSchedulesRequest,
    ListSchedulesResponse,
    PauseScheduleRequest,
    ResumeScheduleRequest,
    UpdateScheduleRequest,
)
from .service_networking import (
    PrivateServiceConnectConfig,
    PscAutomatedEndpoints,
)
from .specialist_pool import (
    SpecialistPool,
)
from .specialist_pool_service import (
    CreateSpecialistPoolOperationMetadata,
    CreateSpecialistPoolRequest,
    DeleteSpecialistPoolRequest,
    GetSpecialistPoolRequest,
    ListSpecialistPoolsRequest,
    ListSpecialistPoolsResponse,
    UpdateSpecialistPoolOperationMetadata,
    UpdateSpecialistPoolRequest,
)
from .study import (
    Measurement,
    Study,
    StudySpec,
    StudyTimeConstraint,
    Trial,
    TrialContext,
)
from .tensorboard import (
    Tensorboard,
)
from .tensorboard_data import (
    Scalar,
    TensorboardBlob,
    TensorboardBlobSequence,
    TensorboardTensor,
    TimeSeriesData,
    TimeSeriesDataPoint,
)
from .tensorboard_experiment import (
    TensorboardExperiment,
)
from .tensorboard_run import (
    TensorboardRun,
)
from .tensorboard_service import (
    BatchCreateTensorboardRunsRequest,
    BatchCreateTensorboardRunsResponse,
    BatchCreateTensorboardTimeSeriesRequest,
    BatchCreateTensorboardTimeSeriesResponse,
    BatchReadTensorboardTimeSeriesDataRequest,
    BatchReadTensorboardTimeSeriesDataResponse,
    CreateTensorboardExperimentRequest,
    CreateTensorboardOperationMetadata,
    CreateTensorboardRequest,
    CreateTensorboardRunRequest,
    CreateTensorboardTimeSeriesRequest,
    DeleteTensorboardExperimentRequest,
    DeleteTensorboardRequest,
    DeleteTensorboardRunRequest,
    DeleteTensorboardTimeSeriesRequest,
    ExportTensorboardTimeSeriesDataRequest,
    ExportTensorboardTimeSeriesDataResponse,
    GetTensorboardExperimentRequest,
    GetTensorboardRequest,
    GetTensorboardRunRequest,
    GetTensorboardTimeSeriesRequest,
    ListTensorboardExperimentsRequest,
    ListTensorboardExperimentsResponse,
    ListTensorboardRunsRequest,
    ListTensorboardRunsResponse,
    ListTensorboardsRequest,
    ListTensorboardsResponse,
    ListTensorboardTimeSeriesRequest,
    ListTensorboardTimeSeriesResponse,
    ReadTensorboardBlobDataRequest,
    ReadTensorboardBlobDataResponse,
    ReadTensorboardSizeRequest,
    ReadTensorboardSizeResponse,
    ReadTensorboardTimeSeriesDataRequest,
    ReadTensorboardTimeSeriesDataResponse,
    ReadTensorboardUsageRequest,
    ReadTensorboardUsageResponse,
    UpdateTensorboardExperimentRequest,
    UpdateTensorboardOperationMetadata,
    UpdateTensorboardRequest,
    UpdateTensorboardRunRequest,
    UpdateTensorboardTimeSeriesRequest,
    WriteTensorboardExperimentDataRequest,
    WriteTensorboardExperimentDataResponse,
    WriteTensorboardRunDataRequest,
    WriteTensorboardRunDataResponse,
)
from .tensorboard_time_series import (
    TensorboardTimeSeries,
)
from .tool import (
    FunctionCall,
    FunctionCallingConfig,
    FunctionDeclaration,
    FunctionResponse,
    GoogleSearchRetrieval,
    Retrieval,
    Tool,
    ToolConfig,
    VertexAISearch,
)
from .training_pipeline import (
    FilterSplit,
    FractionSplit,
    InputDataConfig,
    PredefinedSplit,
    StratifiedSplit,
    TimestampSplit,
    TrainingPipeline,
)
from .tuning_job import (
    SupervisedHyperParameters,
    SupervisedTuningDatasetDistribution,
    SupervisedTuningDataStats,
    SupervisedTuningSpec,
    TunedModel,
    TuningDataStats,
    TuningJob,
)
from .types import (
    BoolArray,
    DoubleArray,
    Int64Array,
    StringArray,
    Tensor,
)
from .unmanaged_container_model import (
    UnmanagedContainerModel,
)
from .user_action_reference import (
    UserActionReference,
)
from .value import (
    Value,
)
from .vizier_service import (
    AddTrialMeasurementRequest,
    CheckTrialEarlyStoppingStateMetatdata,
    CheckTrialEarlyStoppingStateRequest,
    CheckTrialEarlyStoppingStateResponse,
    CompleteTrialRequest,
    CreateStudyRequest,
    CreateTrialRequest,
    DeleteStudyRequest,
    DeleteTrialRequest,
    GetStudyRequest,
    GetTrialRequest,
    ListOptimalTrialsRequest,
    ListOptimalTrialsResponse,
    ListStudiesRequest,
    ListStudiesResponse,
    ListTrialsRequest,
    ListTrialsResponse,
    LookupStudyRequest,
    StopTrialRequest,
    SuggestTrialsMetadata,
    SuggestTrialsRequest,
    SuggestTrialsResponse,
)

__all__ = (
    "AcceleratorType",
    "Annotation",
    "AnnotationSpec",
    "Artifact",
    "BatchPredictionJob",
    "CompletionStats",
    "Blob",
    "Candidate",
    "Citation",
    "CitationMetadata",
    "Content",
    "FileData",
    "GenerationConfig",
    "GroundingChunk",
    "GroundingMetadata",
    "GroundingSupport",
    "LogprobsResult",
    "Part",
    "SafetyRating",
    "SafetySetting",
    "SearchEntryPoint",
    "Segment",
    "VideoMetadata",
    "HarmCategory",
    "Context",
    "ContainerSpec",
    "CustomJob",
    "CustomJobSpec",
    "PythonPackageSpec",
    "Scheduling",
    "WorkerPoolSpec",
    "DataItem",
    "ActiveLearningConfig",
    "DataLabelingJob",
    "SampleConfig",
    "TrainingConfig",
    "Dataset",
    "ExportDataConfig",
    "ExportFilterSplit",
    "ExportFractionSplit",
    "ImportDataConfig",
    "CreateDatasetOperationMetadata",
    "CreateDatasetRequest",
    "CreateDatasetVersionOperationMetadata",
    "CreateDatasetVersionRequest",
    "DataItemView",
    "DeleteDatasetRequest",
    "DeleteDatasetVersionRequest",
    "DeleteSavedQueryRequest",
    "ExportDataOperationMetadata",
    "ExportDataRequest",
    "ExportDataResponse",
    "GetAnnotationSpecRequest",
    "GetDatasetRequest",
    "GetDatasetVersionRequest",
    "ImportDataOperationMetadata",
    "ImportDataRequest",
    "ImportDataResponse",
    "ListAnnotationsRequest",
    "ListAnnotationsResponse",
    "ListDataItemsRequest",
    "ListDataItemsResponse",
    "ListDatasetsRequest",
    "ListDatasetsResponse",
    "ListDatasetVersionsRequest",
    "ListDatasetVersionsResponse",
    "ListSavedQueriesRequest",
    "ListSavedQueriesResponse",
    "RestoreDatasetVersionOperationMetadata",
    "RestoreDatasetVersionRequest",
    "SearchDataItemsRequest",
    "SearchDataItemsResponse",
    "UpdateDatasetRequest",
    "UpdateDatasetVersionRequest",
    "DatasetVersion",
    "DeployedIndexRef",
    "DeployedModelRef",
    "DeploymentResourcePool",
    "CreateDeploymentResourcePoolOperationMetadata",
    "CreateDeploymentResourcePoolRequest",
    "DeleteDeploymentResourcePoolRequest",
    "GetDeploymentResourcePoolRequest",
    "ListDeploymentResourcePoolsRequest",
    "ListDeploymentResourcePoolsResponse",
    "QueryDeployedModelsRequest",
    "QueryDeployedModelsResponse",
    "UpdateDeploymentResourcePoolOperationMetadata",
    "UpdateDeploymentResourcePoolRequest",
    "EncryptionSpec",
    "DeployedModel",
    "Endpoint",
    "PredictRequestResponseLoggingConfig",
    "PrivateEndpoints",
    "CreateEndpointOperationMetadata",
    "CreateEndpointRequest",
    "DeleteEndpointRequest",
    "DeployModelOperationMetadata",
    "DeployModelRequest",
    "DeployModelResponse",
    "GetEndpointRequest",
    "ListEndpointsRequest",
    "ListEndpointsResponse",
    "MutateDeployedModelOperationMetadata",
    "MutateDeployedModelRequest",
    "MutateDeployedModelResponse",
    "UndeployModelOperationMetadata",
    "UndeployModelRequest",
    "UndeployModelResponse",
    "UpdateEndpointRequest",
    "EntityType",
    "EnvVar",
    "ErrorAnalysisAnnotation",
    "EvaluatedAnnotation",
    "EvaluatedAnnotationExplanation",
    "BleuInput",
    "BleuInstance",
    "BleuMetricValue",
    "BleuResults",
    "BleuSpec",
    "CoherenceInput",
    "CoherenceInstance",
    "CoherenceResult",
    "CoherenceSpec",
    "EvaluateInstancesRequest",
    "EvaluateInstancesResponse",
    "ExactMatchInput",
    "ExactMatchInstance",
    "ExactMatchMetricValue",
    "ExactMatchResults",
    "ExactMatchSpec",
    "FluencyInput",
    "FluencyInstance",
    "FluencyResult",
    "FluencySpec",
    "FulfillmentInput",
    "FulfillmentInstance",
    "FulfillmentResult",
    "FulfillmentSpec",
    "GroundednessInput",
    "GroundednessInstance",
    "GroundednessResult",
    "GroundednessSpec",
    "PairwiseMetricInput",
    "PairwiseMetricInstance",
    "PairwiseMetricResult",
    "PairwiseMetricSpec",
    "PairwiseQuestionAnsweringQualityInput",
    "PairwiseQuestionAnsweringQualityInstance",
    "PairwiseQuestionAnsweringQualityResult",
    "PairwiseQuestionAnsweringQualitySpec",
    "PairwiseSummarizationQualityInput",
    "PairwiseSummarizationQualityInstance",
    "PairwiseSummarizationQualityResult",
    "PairwiseSummarizationQualitySpec",
    "PointwiseMetricInput",
    "PointwiseMetricInstance",
    "PointwiseMetricResult",
    "PointwiseMetricSpec",
    "QuestionAnsweringCorrectnessInput",
    "QuestionAnsweringCorrectnessInstance",
    "QuestionAnsweringCorrectnessResult",
    "QuestionAnsweringCorrectnessSpec",
    "QuestionAnsweringHelpfulnessInput",
    "QuestionAnsweringHelpfulnessInstance",
    "QuestionAnsweringHelpfulnessResult",
    "QuestionAnsweringHelpfulnessSpec",
    "QuestionAnsweringQualityInput",
    "QuestionAnsweringQualityInstance",
    "QuestionAnsweringQualityResult",
    "QuestionAnsweringQualitySpec",
    "QuestionAnsweringRelevanceInput",
    "QuestionAnsweringRelevanceInstance",
    "QuestionAnsweringRelevanceResult",
    "QuestionAnsweringRelevanceSpec",
    "RougeInput",
    "RougeInstance",
    "RougeMetricValue",
    "RougeResults",
    "RougeSpec",
    "SafetyInput",
    "SafetyInstance",
    "SafetyResult",
    "SafetySpec",
    "SummarizationHelpfulnessInput",
    "SummarizationHelpfulnessInstance",
    "SummarizationHelpfulnessResult",
    "SummarizationHelpfulnessSpec",
    "SummarizationQualityInput",
    "SummarizationQualityInstance",
    "SummarizationQualityResult",
    "SummarizationQualitySpec",
    "SummarizationVerbosityInput",
    "SummarizationVerbosityInstance",
    "SummarizationVerbosityResult",
    "SummarizationVerbositySpec",
    "ToolCallValidInput",
    "ToolCallValidInstance",
    "ToolCallValidMetricValue",
    "ToolCallValidResults",
    "ToolCallValidSpec",
    "ToolNameMatchInput",
    "ToolNameMatchInstance",
    "ToolNameMatchMetricValue",
    "ToolNameMatchResults",
    "ToolNameMatchSpec",
    "ToolParameterKeyMatchInput",
    "ToolParameterKeyMatchInstance",
    "ToolParameterKeyMatchMetricValue",
    "ToolParameterKeyMatchResults",
    "ToolParameterKeyMatchSpec",
    "ToolParameterKVMatchInput",
    "ToolParameterKVMatchInstance",
    "ToolParameterKVMatchMetricValue",
    "ToolParameterKVMatchResults",
    "ToolParameterKVMatchSpec",
    "PairwiseChoice",
    "Event",
    "Execution",
    "Attribution",
    "BlurBaselineConfig",
    "Examples",
    "ExamplesOverride",
    "ExamplesRestrictionsNamespace",
    "Explanation",
    "ExplanationMetadataOverride",
    "ExplanationParameters",
    "ExplanationSpec",
    "ExplanationSpecOverride",
    "FeatureNoiseSigma",
    "IntegratedGradientsAttribution",
    "ModelExplanation",
    "Neighbor",
    "Presets",
    "SampledShapleyAttribution",
    "SmoothGradConfig",
    "XraiAttribution",
    "ExplanationMetadata",
    "Feature",
    "FeatureGroup",
    "FeatureStatsAnomaly",
    "FeatureOnlineStore",
    "CreateFeatureOnlineStoreOperationMetadata",
    "CreateFeatureOnlineStoreRequest",
    "CreateFeatureViewOperationMetadata",
    "CreateFeatureViewRequest",
    "DeleteFeatureOnlineStoreRequest",
    "DeleteFeatureViewRequest",
    "GetFeatureOnlineStoreRequest",
    "GetFeatureViewRequest",
    "GetFeatureViewSyncRequest",
    "ListFeatureOnlineStoresRequest",
    "ListFeatureOnlineStoresResponse",
    "ListFeatureViewsRequest",
    "ListFeatureViewsResponse",
    "ListFeatureViewSyncsRequest",
    "ListFeatureViewSyncsResponse",
    "SyncFeatureViewRequest",
    "SyncFeatureViewResponse",
    "UpdateFeatureOnlineStoreOperationMetadata",
    "UpdateFeatureOnlineStoreRequest",
    "UpdateFeatureViewOperationMetadata",
    "UpdateFeatureViewRequest",
    "FeatureViewDataKey",
    "FetchFeatureValuesRequest",
    "FetchFeatureValuesResponse",
    "NearestNeighborQuery",
    "NearestNeighbors",
    "SearchNearestEntitiesRequest",
    "SearchNearestEntitiesResponse",
    "FeatureViewDataFormat",
    "CreateFeatureGroupOperationMetadata",
    "CreateFeatureGroupRequest",
    "CreateRegistryFeatureOperationMetadata",
    "DeleteFeatureGroupRequest",
    "GetFeatureGroupRequest",
    "ListFeatureGroupsRequest",
    "ListFeatureGroupsResponse",
    "UpdateFeatureGroupOperationMetadata",
    "UpdateFeatureGroupRequest",
    "UpdateFeatureOperationMetadata",
    "FeatureSelector",
    "IdMatcher",
    "FeatureView",
    "FeatureViewSync",
    "Featurestore",
    "FeaturestoreMonitoringConfig",
    "FeatureValue",
    "FeatureValueList",
    "ReadFeatureValuesRequest",
    "ReadFeatureValuesResponse",
    "StreamingReadFeatureValuesRequest",
    "StructFieldValue",
    "StructValue",
    "WriteFeatureValuesPayload",
    "WriteFeatureValuesRequest",
    "WriteFeatureValuesResponse",
    "BatchCreateFeaturesOperationMetadata",
    "BatchCreateFeaturesRequest",
    "BatchCreateFeaturesResponse",
    "BatchReadFeatureValuesOperationMetadata",
    "BatchReadFeatureValuesRequest",
    "BatchReadFeatureValuesResponse",
    "CreateEntityTypeOperationMetadata",
    "CreateEntityTypeRequest",
    "CreateFeatureOperationMetadata",
    "CreateFeatureRequest",
    "CreateFeaturestoreOperationMetadata",
    "CreateFeaturestoreRequest",
    "DeleteEntityTypeRequest",
    "DeleteFeatureRequest",
    "DeleteFeaturestoreRequest",
    "DeleteFeatureValuesOperationMetadata",
    "DeleteFeatureValuesRequest",
    "DeleteFeatureValuesResponse",
    "DestinationFeatureSetting",
    "EntityIdSelector",
    "ExportFeatureValuesOperationMetadata",
    "ExportFeatureValuesRequest",
    "ExportFeatureValuesResponse",
    "FeatureValueDestination",
    "GetEntityTypeRequest",
    "GetFeatureRequest",
    "GetFeaturestoreRequest",
    "ImportFeatureValuesOperationMetadata",
    "ImportFeatureValuesRequest",
    "ImportFeatureValuesResponse",
    "ListEntityTypesRequest",
    "ListEntityTypesResponse",
    "ListFeaturesRequest",
    "ListFeaturesResponse",
    "ListFeaturestoresRequest",
    "ListFeaturestoresResponse",
    "SearchFeaturesRequest",
    "SearchFeaturesResponse",
    "UpdateEntityTypeRequest",
    "UpdateFeatureRequest",
    "UpdateFeaturestoreOperationMetadata",
    "UpdateFeaturestoreRequest",
    "CancelTuningJobRequest",
    "CreateTuningJobRequest",
    "GetTuningJobRequest",
    "ListTuningJobsRequest",
    "ListTuningJobsResponse",
    "HyperparameterTuningJob",
    "Index",
    "IndexDatapoint",
    "IndexStats",
    "DeployedIndex",
    "DeployedIndexAuthConfig",
    "IndexEndpoint",
    "IndexPrivateEndpoints",
    "CreateIndexEndpointOperationMetadata",
    "CreateIndexEndpointRequest",
    "DeleteIndexEndpointRequest",
    "DeployIndexOperationMetadata",
    "DeployIndexRequest",
    "DeployIndexResponse",
    "GetIndexEndpointRequest",
    "ListIndexEndpointsRequest",
    "ListIndexEndpointsResponse",
    "MutateDeployedIndexOperationMetadata",
    "MutateDeployedIndexRequest",
    "MutateDeployedIndexResponse",
    "UndeployIndexOperationMetadata",
    "UndeployIndexRequest",
    "UndeployIndexResponse",
    "UpdateIndexEndpointRequest",
    "CreateIndexOperationMetadata",
    "CreateIndexRequest",
    "DeleteIndexRequest",
    "GetIndexRequest",
    "ListIndexesRequest",
    "ListIndexesResponse",
    "NearestNeighborSearchOperationMetadata",
    "RemoveDatapointsRequest",
    "RemoveDatapointsResponse",
    "UpdateIndexOperationMetadata",
    "UpdateIndexRequest",
    "UpsertDatapointsRequest",
    "UpsertDatapointsResponse",
    "AvroSource",
    "BigQueryDestination",
    "BigQuerySource",
    "ContainerRegistryDestination",
    "CsvDestination",
    "CsvSource",
    "GcsDestination",
    "GcsSource",
    "TFRecordDestination",
    "CancelBatchPredictionJobRequest",
    "CancelCustomJobRequest",
    "CancelDataLabelingJobRequest",
    "CancelHyperparameterTuningJobRequest",
    "CancelNasJobRequest",
    "CreateBatchPredictionJobRequest",
    "CreateCustomJobRequest",
    "CreateDataLabelingJobRequest",
    "CreateHyperparameterTuningJobRequest",
    "CreateModelDeploymentMonitoringJobRequest",
    "CreateNasJobRequest",
    "DeleteBatchPredictionJobRequest",
    "DeleteCustomJobRequest",
    "DeleteDataLabelingJobRequest",
    "DeleteHyperparameterTuningJobRequest",
    "DeleteModelDeploymentMonitoringJobRequest",
    "DeleteNasJobRequest",
    "GetBatchPredictionJobRequest",
    "GetCustomJobRequest",
    "GetDataLabelingJobRequest",
    "GetHyperparameterTuningJobRequest",
    "GetModelDeploymentMonitoringJobRequest",
    "GetNasJobRequest",
    "GetNasTrialDetailRequest",
    "ListBatchPredictionJobsRequest",
    "ListBatchPredictionJobsResponse",
    "ListCustomJobsRequest",
    "ListCustomJobsResponse",
    "ListDataLabelingJobsRequest",
    "ListDataLabelingJobsResponse",
    "ListHyperparameterTuningJobsRequest",
    "ListHyperparameterTuningJobsResponse",
    "ListModelDeploymentMonitoringJobsRequest",
    "ListModelDeploymentMonitoringJobsResponse",
    "ListNasJobsRequest",
    "ListNasJobsResponse",
    "ListNasTrialDetailsRequest",
    "ListNasTrialDetailsResponse",
    "PauseModelDeploymentMonitoringJobRequest",
    "ResumeModelDeploymentMonitoringJobRequest",
    "SearchModelDeploymentMonitoringStatsAnomaliesRequest",
    "SearchModelDeploymentMonitoringStatsAnomaliesResponse",
    "UpdateModelDeploymentMonitoringJobOperationMetadata",
    "UpdateModelDeploymentMonitoringJobRequest",
    "JobState",
    "LineageSubgraph",
    "ComputeTokensRequest",
    "ComputeTokensResponse",
    "TokensInfo",
    "AutomaticResources",
    "AutoscalingMetricSpec",
    "BatchDedicatedResources",
    "DedicatedResources",
    "DiskSpec",
    "MachineSpec",
    "NfsMount",
    "PersistentDiskSpec",
    "ResourcesConsumed",
    "ShieldedVmConfig",
    "ManualBatchTuningParameters",
    "FindNeighborsRequest",
    "FindNeighborsResponse",
    "ReadIndexDatapointsRequest",
    "ReadIndexDatapointsResponse",
    "MetadataSchema",
    "AddContextArtifactsAndExecutionsRequest",
    "AddContextArtifactsAndExecutionsResponse",
    "AddContextChildrenRequest",
    "AddContextChildrenResponse",
    "AddExecutionEventsRequest",
    "AddExecutionEventsResponse",
    "CreateArtifactRequest",
    "CreateContextRequest",
    "CreateExecutionRequest",
    "CreateMetadataSchemaRequest",
    "CreateMetadataStoreOperationMetadata",
    "CreateMetadataStoreRequest",
    "DeleteArtifactRequest",
    "DeleteContextRequest",
    "DeleteExecutionRequest",
    "DeleteMetadataStoreOperationMetadata",
    "DeleteMetadataStoreRequest",
    "GetArtifactRequest",
    "GetContextRequest",
    "GetExecutionRequest",
    "GetMetadataSchemaRequest",
    "GetMetadataStoreRequest",
    "ListArtifactsRequest",
    "ListArtifactsResponse",
    "ListContextsRequest",
    "ListContextsResponse",
    "ListExecutionsRequest",
    "ListExecutionsResponse",
    "ListMetadataSchemasRequest",
    "ListMetadataSchemasResponse",
    "ListMetadataStoresRequest",
    "ListMetadataStoresResponse",
    "PurgeArtifactsMetadata",
    "PurgeArtifactsRequest",
    "PurgeArtifactsResponse",
    "PurgeContextsMetadata",
    "PurgeContextsRequest",
    "PurgeContextsResponse",
    "PurgeExecutionsMetadata",
    "PurgeExecutionsRequest",
    "PurgeExecutionsResponse",
    "QueryArtifactLineageSubgraphRequest",
    "QueryContextLineageSubgraphRequest",
    "QueryExecutionInputsAndOutputsRequest",
    "RemoveContextChildrenRequest",
    "RemoveContextChildrenResponse",
    "UpdateArtifactRequest",
    "UpdateContextRequest",
    "UpdateExecutionRequest",
    "MetadataStore",
    "MigratableResource",
    "BatchMigrateResourcesOperationMetadata",
    "BatchMigrateResourcesRequest",
    "BatchMigrateResourcesResponse",
    "MigrateResourceRequest",
    "MigrateResourceResponse",
    "SearchMigratableResourcesRequest",
    "SearchMigratableResourcesResponse",
    "GenieSource",
    "LargeModelReference",
    "Model",
    "ModelContainerSpec",
    "ModelGardenSource",
    "ModelSourceInfo",
    "Port",
    "PredictSchemata",
    "Probe",
    "ModelDeploymentMonitoringBigQueryTable",
    "ModelDeploymentMonitoringJob",
    "ModelDeploymentMonitoringObjectiveConfig",
    "ModelDeploymentMonitoringScheduleConfig",
    "ModelMonitoringStatsAnomalies",
    "ModelDeploymentMonitoringObjectiveType",
    "ModelEvaluation",
    "ModelEvaluationSlice",
    "GetPublisherModelRequest",
    "PublisherModelView",
    "ModelMonitoringAlertConfig",
    "ModelMonitoringObjectiveConfig",
    "SamplingStrategy",
    "ThresholdConfig",
    "BatchImportEvaluatedAnnotationsRequest",
    "BatchImportEvaluatedAnnotationsResponse",
    "BatchImportModelEvaluationSlicesRequest",
    "BatchImportModelEvaluationSlicesResponse",
    "CopyModelOperationMetadata",
    "CopyModelRequest",
    "CopyModelResponse",
    "DeleteModelRequest",
    "DeleteModelVersionRequest",
    "ExportModelOperationMetadata",
    "ExportModelRequest",
    "ExportModelResponse",
    "GetModelEvaluationRequest",
    "GetModelEvaluationSliceRequest",
    "GetModelRequest",
    "ImportModelEvaluationRequest",
    "ListModelEvaluationSlicesRequest",
    "ListModelEvaluationSlicesResponse",
    "ListModelEvaluationsRequest",
    "ListModelEvaluationsResponse",
    "ListModelsRequest",
    "ListModelsResponse",
    "ListModelVersionsRequest",
    "ListModelVersionsResponse",
    "MergeVersionAliasesRequest",
    "UpdateExplanationDatasetOperationMetadata",
    "UpdateExplanationDatasetRequest",
    "UpdateExplanationDatasetResponse",
    "UpdateModelRequest",
    "UploadModelOperationMetadata",
    "UploadModelRequest",
    "UploadModelResponse",
    "NasJob",
    "NasJobOutput",
    "NasJobSpec",
    "NasTrial",
    "NasTrialDetail",
    "NetworkSpec",
    "NotebookEucConfig",
    "NotebookExecutionJob",
    "NotebookIdleShutdownConfig",
    "NotebookRuntime",
    "NotebookRuntimeTemplate",
    "NotebookRuntimeType",
    "NotebookRuntimeTemplateRef",
    "AssignNotebookRuntimeOperationMetadata",
    "AssignNotebookRuntimeRequest",
    "CreateNotebookExecutionJobOperationMetadata",
    "CreateNotebookExecutionJobRequest",
    "CreateNotebookRuntimeTemplateOperationMetadata",
    "CreateNotebookRuntimeTemplateRequest",
    "DeleteNotebookExecutionJobRequest",
    "DeleteNotebookRuntimeRequest",
    "DeleteNotebookRuntimeTemplateRequest",
    "GetNotebookExecutionJobRequest",
    "GetNotebookRuntimeRequest",
    "GetNotebookRuntimeTemplateRequest",
    "ListNotebookExecutionJobsRequest",
    "ListNotebookExecutionJobsResponse",
    "ListNotebookRuntimesRequest",
    "ListNotebookRuntimesResponse",
    "ListNotebookRuntimeTemplatesRequest",
    "ListNotebookRuntimeTemplatesResponse",
    "StartNotebookRuntimeOperationMetadata",
    "StartNotebookRuntimeRequest",
    "StartNotebookRuntimeResponse",
    "UpdateNotebookRuntimeTemplateRequest",
    "UpgradeNotebookRuntimeOperationMetadata",
    "UpgradeNotebookRuntimeRequest",
    "UpgradeNotebookRuntimeResponse",
    "NotebookExecutionJobView",
    "Schema",
    "Type",
    "DeleteOperationMetadata",
    "GenericOperationMetadata",
    "PersistentResource",
    "RayLogsSpec",
    "RayMetricSpec",
    "RaySpec",
    "ResourcePool",
    "ResourceRuntime",
    "ResourceRuntimeSpec",
    "ServiceAccountSpec",
    "CreatePersistentResourceOperationMetadata",
    "CreatePersistentResourceRequest",
    "DeletePersistentResourceRequest",
    "GetPersistentResourceRequest",
    "ListPersistentResourcesRequest",
    "ListPersistentResourcesResponse",
    "RebootPersistentResourceOperationMetadata",
    "RebootPersistentResourceRequest",
    "UpdatePersistentResourceOperationMetadata",
    "UpdatePersistentResourceRequest",
    "PipelineFailurePolicy",
    "PipelineJob",
    "PipelineJobDetail",
    "PipelineTaskDetail",
    "PipelineTaskExecutorDetail",
    "PipelineTemplateMetadata",
    "BatchCancelPipelineJobsOperationMetadata",
    "BatchCancelPipelineJobsRequest",
    "BatchCancelPipelineJobsResponse",
    "BatchDeletePipelineJobsRequest",
    "BatchDeletePipelineJobsResponse",
    "CancelPipelineJobRequest",
    "CancelTrainingPipelineRequest",
    "CreatePipelineJobRequest",
    "CreateTrainingPipelineRequest",
    "DeletePipelineJobRequest",
    "DeleteTrainingPipelineRequest",
    "GetPipelineJobRequest",
    "GetTrainingPipelineRequest",
    "ListPipelineJobsRequest",
    "ListPipelineJobsResponse",
    "ListTrainingPipelinesRequest",
    "ListTrainingPipelinesResponse",
    "PipelineState",
    "CountTokensRequest",
    "CountTokensResponse",
    "DirectPredictRequest",
    "DirectPredictResponse",
    "DirectRawPredictRequest",
    "DirectRawPredictResponse",
    "ExplainRequest",
    "ExplainResponse",
    "GenerateContentRequest",
    "GenerateContentResponse",
    "PredictRequest",
    "PredictResponse",
    "RawPredictRequest",
    "StreamDirectPredictRequest",
    "StreamDirectPredictResponse",
    "StreamDirectRawPredictRequest",
    "StreamDirectRawPredictResponse",
    "StreamingPredictRequest",
    "StreamingPredictResponse",
    "StreamingRawPredictRequest",
    "StreamingRawPredictResponse",
    "StreamRawPredictRequest",
    "PublisherModel",
    "ReservationAffinity",
    "SavedQuery",
    "Schedule",
    "CreateScheduleRequest",
    "DeleteScheduleRequest",
    "GetScheduleRequest",
    "ListSchedulesRequest",
    "ListSchedulesResponse",
    "PauseScheduleRequest",
    "ResumeScheduleRequest",
    "UpdateScheduleRequest",
    "PrivateServiceConnectConfig",
    "PscAutomatedEndpoints",
    "SpecialistPool",
    "CreateSpecialistPoolOperationMetadata",
    "CreateSpecialistPoolRequest",
    "DeleteSpecialistPoolRequest",
    "GetSpecialistPoolRequest",
    "ListSpecialistPoolsRequest",
    "ListSpecialistPoolsResponse",
    "UpdateSpecialistPoolOperationMetadata",
    "UpdateSpecialistPoolRequest",
    "Measurement",
    "Study",
    "StudySpec",
    "StudyTimeConstraint",
    "Trial",
    "TrialContext",
    "Tensorboard",
    "Scalar",
    "TensorboardBlob",
    "TensorboardBlobSequence",
    "TensorboardTensor",
    "TimeSeriesData",
    "TimeSeriesDataPoint",
    "TensorboardExperiment",
    "TensorboardRun",
    "BatchCreateTensorboardRunsRequest",
    "BatchCreateTensorboardRunsResponse",
    "BatchCreateTensorboardTimeSeriesRequest",
    "BatchCreateTensorboardTimeSeriesResponse",
    "BatchReadTensorboardTimeSeriesDataRequest",
    "BatchReadTensorboardTimeSeriesDataResponse",
    "CreateTensorboardExperimentRequest",
    "CreateTensorboardOperationMetadata",
    "CreateTensorboardRequest",
    "CreateTensorboardRunRequest",
    "CreateTensorboardTimeSeriesRequest",
    "DeleteTensorboardExperimentRequest",
    "DeleteTensorboardRequest",
    "DeleteTensorboardRunRequest",
    "DeleteTensorboardTimeSeriesRequest",
    "ExportTensorboardTimeSeriesDataRequest",
    "ExportTensorboardTimeSeriesDataResponse",
    "GetTensorboardExperimentRequest",
    "GetTensorboardRequest",
    "GetTensorboardRunRequest",
    "GetTensorboardTimeSeriesRequest",
    "ListTensorboardExperimentsRequest",
    "ListTensorboardExperimentsResponse",
    "ListTensorboardRunsRequest",
    "ListTensorboardRunsResponse",
    "ListTensorboardsRequest",
    "ListTensorboardsResponse",
    "ListTensorboardTimeSeriesRequest",
    "ListTensorboardTimeSeriesResponse",
    "ReadTensorboardBlobDataRequest",
    "ReadTensorboardBlobDataResponse",
    "ReadTensorboardSizeRequest",
    "ReadTensorboardSizeResponse",
    "ReadTensorboardTimeSeriesDataRequest",
    "ReadTensorboardTimeSeriesDataResponse",
    "ReadTensorboardUsageRequest",
    "ReadTensorboardUsageResponse",
    "UpdateTensorboardExperimentRequest",
    "UpdateTensorboardOperationMetadata",
    "UpdateTensorboardRequest",
    "UpdateTensorboardRunRequest",
    "UpdateTensorboardTimeSeriesRequest",
    "WriteTensorboardExperimentDataRequest",
    "WriteTensorboardExperimentDataResponse",
    "WriteTensorboardRunDataRequest",
    "WriteTensorboardRunDataResponse",
    "TensorboardTimeSeries",
    "FunctionCall",
    "FunctionCallingConfig",
    "FunctionDeclaration",
    "FunctionResponse",
    "GoogleSearchRetrieval",
    "Retrieval",
    "Tool",
    "ToolConfig",
    "VertexAISearch",
    "FilterSplit",
    "FractionSplit",
    "InputDataConfig",
    "PredefinedSplit",
    "StratifiedSplit",
    "TimestampSplit",
    "TrainingPipeline",
    "SupervisedHyperParameters",
    "SupervisedTuningDatasetDistribution",
    "SupervisedTuningDataStats",
    "SupervisedTuningSpec",
    "TunedModel",
    "TuningDataStats",
    "TuningJob",
    "BoolArray",
    "DoubleArray",
    "Int64Array",
    "StringArray",
    "Tensor",
    "UnmanagedContainerModel",
    "UserActionReference",
    "Value",
    "AddTrialMeasurementRequest",
    "CheckTrialEarlyStoppingStateMetatdata",
    "CheckTrialEarlyStoppingStateRequest",
    "CheckTrialEarlyStoppingStateResponse",
    "CompleteTrialRequest",
    "CreateStudyRequest",
    "CreateTrialRequest",
    "DeleteStudyRequest",
    "DeleteTrialRequest",
    "GetStudyRequest",
    "GetTrialRequest",
    "ListOptimalTrialsRequest",
    "ListOptimalTrialsResponse",
    "ListStudiesRequest",
    "ListStudiesResponse",
    "ListTrialsRequest",
    "ListTrialsResponse",
    "LookupStudyRequest",
    "StopTrialRequest",
    "SuggestTrialsMetadata",
    "SuggestTrialsRequest",
    "SuggestTrialsResponse",
)
